/*
 * MP3/MPlayer plugin to VDR (C++)
 *
 * (C) 2001,2002 Stefan Huelswitt <huels@iname.com>
 *
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 */

#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include "common.h"
#include "data-mp3.h"
#include "data.h"
#include "decoder.h"

cFileSources MP3Sources;

// --- cSong -------------------------------------------------------------

cSong::cSong(const char *Name, cFileSource *Source)
{
  decoder=0; name=fullname=0; source=0;
  fromDOS=decoderFailed=false;
  source=Source;
  if(Name) Set(Name);
  source->Block();
}

cSong::~cSong()
{
  Clear();
  source->Unblock();
}

void cSong::Set(const char *Name)
{
  Clear();
  name=strdup(Name);
  fullname=source->BuildName(name);
}

void cSong::Clear(void)
{
  delete decoder; decoder=0;
  free(name); name=0;
  free(fullname); fullname=0;
}

cSongInfo *cSong::Info(bool get)
{
  Decoder();
  cSongInfo *si=0;
  if(decoder) si=decoder->SongInfo(get);
  return si;
}

cDecoder *cSong::Decoder(void)
{
  decLock.Lock();
  if(!decoder && !decoderFailed) {
    decoder=cDecoders::FindDecoder(name,source);
    if(!decoder) decoderFailed=true;
    }
  decLock.Unlock();
  return decoder;
}

void cSong::Convert(void)
{
  char *Name=Convert2Unix();
  Set(Name);
  fromDOS=true;
  free(Name);
}

char *cSong::Convert2Unix(void)
{
  char *Name=strdup(name);
  char *p=Name;
  while(*p) {
    if(*p=='/') *p='?';
    if(*p=='\\') *p='/';
    p++;
    }
  return Name;
}

char *cSong::Convert2Dos(void)
{
/*
  char *Name=strdup(name);
  char *p=Name;
  while(*p) {
    if(*p=='\\') *p='?';
    if(*p=='/') *p='\\';
    p++;
    }
  return Name;
*/
  return 0;
}

bool cSong::Parse(char *s)
{
  s = skipspace(stripspace(s));
  if(*s) { Set(s); return true; }
  return false;
}

bool cSong::Save(FILE *f)
{
  int res;
/*
  if(fromDOS) {
    char *name=Convert2Dos();
    res=fprintf(f,"%s\n",name);
    free(name);
    }
  else
*/
    res=fprintf(f,"%s\n",name);
  return res>0;
}

// -- cPlayList --------------------------------------------------------------

cPlayList::cPlayList(const char *Name, cFileSource *Source)
{
  extbuffer=name=basename=fullname=0;
  isWinAmp=false;
  source=Source;
  source->Block();
  if(Name) Set(Name);
}

cPlayList::~cPlayList()
{
  free(name);
  free(basename);
  free(fullname);
  free(extbuffer);
  source->Unblock();
}

void cPlayList::Set(const char *Name)
{
  free(name);
  name=strdup(Name);
  free(basename);
  basename=strdup(name);
  int l=strlen(basename)-strlen(PLAYLISTEXT);
  if(l>0 && !strcasecmp(basename+l,PLAYLISTEXT)) basename[l]=0;
  free(fullname);
  fullname=source->BuildName(name);
}

bool cPlayList::Load(void)
{
  Clear();
  bool result=false;
  if(fullname) {
    FILE *f=fopen(fullname,"r");
    if(f) {
      char buffer[512];
      result=true;
      while(fgets(buffer,sizeof(buffer),f)>0) {
        if(buffer[0]=='#') {
          if(!strncmp(buffer,WINAMPEXT,strlen(WINAMPEXT))) {
            d(printf("mp3: detected WinAmp style playlist\n"))
            isWinAmp=true;
            }
          continue;
          }
        if(!isempty(buffer)) {
          cSong *song=new cSong(0,source);
          if(song->Parse(buffer)) Add(song);
          else {
            esyslog("error loading playlist %s\n",fullname);
            delete song;
            result=false;
            break;
            }
          }
        }
      fclose(f);
      }
    else LOG_ERROR_STR(fullname);

    if(result && isWinAmp) {
      cSong *song=First();
      while(song) {   // if this is a WinAmp playlist, convert \ to /
        song->Convert();
        song=cList<cSong>::Next(song);
        }
      }
    }
  return result;
}

bool cPlayList::Save(void)
{
  bool result=true;
  cSafeFile f(fullname);
  if(f.Open()) {
    cSong *song=First();
    while(song) {
      if(!song->Save(f)) {
         result=false;
         break;
         }
      song=cList<cSong>::Next(song);
      }
    if(!f.Close()) result=false;
    }
  else result=false;
  return result;
}
 
bool cPlayList::Exists(void)
{
  struct stat64 ds;
  if(!stat64(fullname,&ds) && S_ISREG(ds.st_mode) && !access(fullname,R_OK)) return true;
  return false;
}

bool cPlayList::TestName(const char *newName)
{
  char *n=source->BuildName(AddExt(newName,PLAYLISTEXT));
  int r=access(n,F_OK);
  free(n);
  return (r==0);
}

bool cPlayList::Rename(const char *newName)
{
  const char *fname=AddExt(newName,PLAYLISTEXT);
  char *nfull=source->BuildName(fname);
  bool r=false;
  if(access(nfull,F_OK) && (!rename(fullname,nfull))) {
    Set(fname); r=true;
    }
  free(nfull);
  return r;
}

bool cPlayList::Create(const char *newName)
{
  const char *fname=AddExt(newName,PLAYLISTEXT);
  char *full=source->BuildName(fname);
  FILE *newf;
  bool r=false;
  if(access(full,F_OK) && (newf=fopen(full,"w"))) {
    fclose(newf);
    Set(fname);
    r=Load();
    }
  free(full);
  return r;
}

bool cPlayList::Delete(void)
{
  if(!unlink(fullname)) return true;
  return false;
}

const char *cPlayList::AddExt(const char *FileName, const char *Ext)
{
  free(extbuffer); extbuffer=0;
  asprintf(&extbuffer,"%s%s",FileName,Ext);
  return extbuffer;
}

// -- cInstantPlayList ------------------------------------------------------

cInstantPlayList::cInstantPlayList(cDirItem *Item)
:cPlayList(Item->Name?Item->Name:"instant",Item->Source)
,item(Item->Source,Item->Subdir,Item->Name,Item->Type)
{}

bool cInstantPlayList::Load(void)
{
  bool res=false;
  Clear();
  switch(item.Type) {
    case itFile:
      d(printf("instant: file %s\n",item.Name))
      DoItem(item.Source,item.Subdir,item.Name);
      res=true; break;
    case itDir:
      {
      d(printf("instant: dir %s\n",item.Name))
      char *path=item.Path();
      res=ScanDir(item.Source,path,stFile,item.Source->Include(),"*"PLAYLISTEXT,true);
      free(path);
      break;
      }
    case itBase:
      d(printf("instant: base\n"))
      res=ScanDir(item.Source,0,stFile,item.Source->Include(),"*"PLAYLISTEXT,true);
      break;
    default: break;
    }
  return res;
}

void cInstantPlayList::DoItem(cFileSource *src, const char *subdir, const char *name)
{
  char *path=(char *)name;
  if(subdir) path=AddPath(subdir,name);
  if (strlen (path) > 4)
  {
    char *suffix = &path[strlen (path) - 3];
    if (strcasecmp (suffix, "gif") && strcasecmp (suffix, "jpg") &&
        strcasecmp (suffix, "png"))
      Add(new cSong(path,src));
  }
  if(subdir) free(path);
}

// -- cPlayLists --------------------------------------------------------------

bool cPlayLists::Load(cFileSource *Source)
{
  Clear();
  return ScanDir(Source,0,stFile,"*"PLAYLISTEXT,0,false);
}

void cPlayLists::DoItem(cFileSource *src, const char *subdir, const char *name)
{
  char *path=(char *)name;
  if(subdir) path=AddPath(subdir,name);  
  Add(new cPlayList(path,src));
  if(subdir) free(path);
}
